#### <sub><sup><a name="v0730-note-1" href="#v0730-note-1">:link:</a></sup></sub> feature, breaking

* The [`fly destroy-pipeline`](https://concourse-ci.org/managing-pipelines.html#fly-destroy-pipeline) command now runs much quicker. Unfortunately to implement this there's a massive database migration. Expect anywhere from a few minutes of downtime up to a few hours when you upgrade to this version, depending on how many builds you have and how chatty they are. Sorry about that.
  
  There is no way of predicting how long this migration may take for your instance. It depends on the chattiness of your builds and the performance of your database. For a rough approximation: running the migration on the Concourse team's server took 20 minutes to migrate 13 million build events.
  
  You can find out how many build events you have by running the following query against your Concourse database:
  
  ```sql
  SELECT relname, n_live_tup
  FROM pg_stat_user_tables
  WHERE relname = 'build_events';
  ```
  
  The root of the issue is the amount of data in the `build_events` table. If you don't have many build logs then then you can probably just upgrade and not worry about the rest of this. If the upgrade is going to take too long then you'll need to find some way to reduce the rows in that table.
  
  Before you delete any data from the system you should make sure to take a [backup of your database](http://www.postgresql.org/docs/9.1/static/backup.html) and make doubly sure you can restore it while blind-folded and upside-down in case anything goes wrong.
  
  If you don't care about your old build logs then you can simply run `TRUNCATE build_events;` before upgrading and the migration will be quick and painless.
  
  Many of you probably do care about your build events but maybe only those which were created in the past X months. If this sounds like a good idea then have I got the SQL query for you! Run this against your Concourse database (change the X to the number of months you'd like to keep):
  
  ```sql
  DELETE FROM build_events
  WHERE build_id IN (
    SELECT builds.id
    FROM jobs JOIN builds ON jobs.id = builds.job_id
    WHERE builds.end_time < NOW() - INTERVAL 'X month'
    ORDER BY builds.id
  );
  ```
  
  This query may take a while to execute but your Concourse can be online the entire time that it is running.
  
  If you want to keep all of your build logs and have a fast migration then I'm sorry, I can't help you. :( Maybe upgrade over a weekend?
  
  
#### <sub><sup><a name="v0730-note-2" href="#v0730-note-2">:link:</a></sup></sub> breaking

* The [Docker Image resource](https://github.com/concourse/docker-image-resource) no longer produces the `docker save`d image by default. This is to reduce disk usage when using the resource as an [**`image_resource`**](https://concourse-ci.org/tasks.html#image_resource). You must now pass `save: true` as part of `params` on the [`get` step](https://concourse-ci.org/get-step.html) to produce the `image` file.
  
  
#### <sub><sup><a name="v0730-note-3" href="#v0730-note-3">:link:</a></sup></sub> feature

* Inputs to jobs that are not configured to [**`trigger`**](https://concourse-ci.org/get-step.html#get-step-trigger) the job when new versions appear will now be rendered with a dashed line. This makes it easier to see which resources automate the pipeline flow, and which jobs are only ever manually triggered.
  
  
#### <sub><sup><a name="v0730-note-4" href="#v0730-note-4">:link:</a></sup></sub> feature

* A pending build will now indicate why it's pending, via a checklist that appears at the top of the build output. (Yay!)
  
  
#### <sub><sup><a name="v0730-note-5" href="#v0730-note-5">:link:</a></sup></sub> breaking

* The theme selector is gone. So are all but one of the themes. This new theme is the product of our research and your feedback. I'm confident that it's perfect in every way. But let us know if you have any major problems with it.
  
  
#### <sub><sup><a name="v0730-note-6" href="#v0730-note-6">:link:</a></sup></sub> fix

* Improved the caching of resources used for [**`image_resource`**](https://concourse-ci.org/tasks.html#image_resource). Previously if the same version was fetched multiple times on the same worker, we'd keep all of them around so long as they were the latest version. We'll now only keep one.
  
  
#### <sub><sup><a name="v0730-note-7" href="#v0730-note-7">:link:</a></sup></sub> feature

* The [`fly destroy-pipeline`](https://concourse-ci.org/managing-pipelines.html#fly-destroy-pipeline) command learnt the `-n` option which when used will not ask you to confirm the deletion of the pipeline. Useful for scripts. Dangerous for users.
  
  
#### <sub><sup><a name="v0730-note-8" href="#v0730-note-8">:link:</a></sup></sub> fix

* The [Docker Image resource](https://github.com/concourse/docker-image-resource) no longer worked for images configured with `ENTRYPOINT`s as of v0.72.0, and ended up running whatever the entrypoint was, with our internal binary tool as an argument. It now works again. Our bad.
  
  
#### <sub><sup><a name="v0730-note-9" href="#v0730-note-9">:link:</a></sup></sub> fix

* The [GitHub release resource](https://github.com/concourse/github-release-resource) would have issues when fetching artefacts from S3 via GitHub when using an access token. It no longer has these issues.
  
  
#### <sub><sup><a name="v0730-note-10" href="#v0730-note-10">:link:</a></sup></sub> feature

* The [Docker Image resource](https://github.com/concourse/docker-image-resource) now supports a `dockerfile` parameter for specifying a path to the `Dockerfile` to use.
  
  
#### <sub><sup><a name="v0730-note-11" href="#v0730-note-11">:link:</a></sup></sub> feature

* The [Git resource](https://github.com/concourse/git-resource) now supports producing annotated tags via the `annotate` parameter.
  
  
#### <sub><sup><a name="v0730-note-12" href="#v0730-note-12">:link:</a></sup></sub> feature

* The [Git resource](https://github.com/concourse/git-resource) now supports checking for tag patterns like `*-production` via the `tag_filter` source configuration.
  
  
#### <sub><sup><a name="v0730-note-13" href="#v0730-note-13">:link:</a></sup></sub> feature

* The [Git resource](https://github.com/concourse/git-resource) now includes [`git lfs`](https://git-lfs.github.com/).
  
  
