#### <sub><sup><a name="v310-note-1" href="#v310-note-1">:link:</a></sup></sub> feature, breaking

* Resources are now `unprivileged` by default. We've been wanting to do this for a while for security reasons (especially in the context of custom resource types), but we ended up needing to do it anyway as part of the change to `overlay` (two release notes down), in kind of a roundabout way.
  
  To make a custom resource type privileged, configure [`privileged: true`](https://concourse-ci.org/resource-types.html#resource-type-privileged).
  
  This change should only require action from folks using a custom resource type that actually has depended on being privileged, such as any forks of the [`docker-image` resource](https://github.com/concourse/docker-image-resource). It may be that you (or the resource author) will not know this until it fails. Resource authors are encouraged to update their READMEs if their resource type does indeed need it.
  
  This was motivated by the one downside of `overlay` that we encountered: namespacing a volume takes much longer. Namespacing a volume means re-mapping its files to have UIDs that correspond to the correct UIDs for its target container's user namespace. This typically a recursive `chown` that remaps host-side UID 0 to the container's equivalent (UID 4 billion something), or vice-versa.
  
  Namespacing a volume takes longer for `overlay` because a `chown` of a file necessitates copying the file to the upper layer, which takes time and space. This did not affect `btrfs` as the nature of its copy-on-write mechanism is vastly different.
  
  Most workloads should be unprivileged, so to optimize for that, we've made all resources unprivileged by default. This means that they generate unprivileged volumes which won't need to be namespaced for unprivileged tasks (and other resources, i.e. `put`).
  
  
#### <sub><sup><a name="v310-note-2" href="#v310-note-2">:link:</a></sup></sub> feature

* Pipeline config and team auth settings can now be encrypted in the database. See [Encryption](https://concourse-ci.org/encryption.html) for more information.
  
  This is the first step towards a better security model around credential management. The next step will be to externalize credentials entirely to a credential manager such as Vault or CredHub.
  
  
#### <sub><sup><a name="v310-note-3" href="#v310-note-3">:link:</a></sup></sub> feature

* Workers will now use `overlay` rather than `btrfs` as their filesystem of choice for BaggageClaim.
  
  If you're morbidly curious about all the low-level mumbo jumbo that went into this venture, check out issue concourse/concourse#1045.
  
  If you don't really care, know that upon upgrade the workers will stick with `btrfs`, for backwards-compatibility. It is only on recreate that they will pick `overlay`, and only if it makes sense to (i.e. your kernel version is > 4.0, and your disk is not formatted as `btrfs` already).
  
  There are many benefits to what sounds like a low-level change:
  
  
  
  * No more loopback shenanigans. Most of you aren't using `btrfs` as your disk's filesystem, so Concourse would always have to make a sparse file and mount a loopback device over it. This was just confusing and made disk usage hard to interpret, and possibly leak.
    
    
  
  * No more locked-up workers! Many users, including us, encountered a `btrfs` bug which led to reads and writes of the disk locking up. The only way out from this was to recreate the worker.
    
    
  
  * Concourse in Docker for Mac should now work! They had `btrfs` stripped out of the kernel image, so Concourse had to fall back on the `naive` driver, which is hella slow.
    
    
  
  * A noticeable performance boost under load. In our testing, `overlay` performs better under realstic work loads like building many Docker images at once. While `overlay` would edge out `btrfs` at low contention, it performed even better as we ramped up the concurrency. What took `btrfs` 10 minutes would take `overlay` around 7.
    
    Also worth noting is that during testing we could pretty reliably get `btrfs` to lock up. It actually made it hard to collect enough data. But maybe that's a more important point than any set of numbers.
    
    
  
  
#### <sub><sup><a name="v310-note-4" href="#v310-note-4">:link:</a></sup></sub> fix

* Fixed metadata for fetched resources often not being shown for resources configured in multiple pipelines, due to the globalization of resource caches introduced by [**v3.0.0**](https://github.com/concourse/concourse/releases/tag/v3.0.0).
  
  
#### <sub><sup><a name="v310-note-5" href="#v310-note-5">:link:</a></sup></sub> feature

* [`fly` CLI](https://concourse-ci.org/fly.html) now supports tab completion for `-t`, thanks to a PR by @jmcarp!
  
  
#### <sub><sup><a name="v310-note-6" href="#v310-note-6">:link:</a></sup></sub> feature

* [`fly set-pipeline`](https://concourse-ci.org/setting-pipelines.html#fly-set-pipeline) now supports tab completion for `-c`, thanks to a PR by @jmcarp!
  
  
#### <sub><sup><a name="v310-note-7" href="#v310-note-7">:link:</a></sup></sub> feature

* Added a timeout when connecting to Postgres.
  
  
#### <sub><sup><a name="v310-note-8" href="#v310-note-8">:link:</a></sup></sub> fix

* Fixed running `concourse web` on Windows. This was caused by building it with an out-of-date version of Go. We've now bumped to the latest.
  
  
#### <sub><sup><a name="v310-note-9" href="#v310-note-9">:link:</a></sup></sub> fix

* Bumped the BaggageClaim response header timeout to 10 minutes, up from 1. The point of this value was more to be "not infinite", so that ought to be enough to account for slower disks.
  
  
#### <sub><sup><a name="v310-note-10" href="#v310-note-10">:link:</a></sup></sub> fix

* Fixed overriding a base resource type with a custom resource type of the same name.
  
  
#### <sub><sup><a name="v310-note-11" href="#v310-note-11">:link:</a></sup></sub> feature

* The legend in the pipeline UI will now show the meaning for dashed vs. solid lines, thanks to a PR by @Typositoire!
  
  
#### <sub><sup><a name="v310-note-12" href="#v310-note-12">:link:</a></sup></sub> feature

* The `concourse/lite` Vagrant box will now respect any locally configured proxy, thanks to a PR by @akumria!
  
  
#### <sub><sup><a name="v310-note-13" href="#v310-note-13">:link:</a></sup></sub> fix

* The ATC will uses `<a>` instead of `<span>` for a few clickable elements, so that things like Vimperator or Vimium can target them. This is thanks to a PR by @dolph!
  
  
#### <sub><sup><a name="v310-note-14" href="#v310-note-14">:link:</a></sup></sub> feature

* The [`semver` resource](https://github.com/concourse/semver-resource) will now ignore leading/trailing whitespace in the contents of the file in the bucket, thanks to a PR by @jghiloni! This would usually happen when manually adding the file, as a trailing linebreak is a common default for most editors (or even `echo`).
  
  
#### <sub><sup><a name="v310-note-15" href="#v310-note-15">:link:</a></sup></sub> fix

* The [`docker-image` resource](https://github.com/concourse/docker-image-resource) should now correctly handle fully numeric tags rather than forcing the user to quote them, thanks to a PR by @benmoss!
  
  
